@@ -2,20 +2,11 @@
 # $Id: bench.pl 1126 2011-10-31 19:56:35Z tinita $
 use strict;
 use warnings;
-use lib qw(blib/lib ../blib/lib);
 use Getopt::Long;
 use FindBin qw/ $RealBin /;
 chdir "$RealBin/..";
 #use Devel::Size qw(size total_size);
 my $count = 0;
-my $ht_file = 'test.htc';
-#$ht_file = 'test.htc.10';
-#$ht_file = 'test.htc.20';
-my $htcc_file = $ht_file . 'c';
-my $tt_file = "test.tt";
-#$tt_file = "test.tt.10";
-#$tt_file = "test.tt.20";
-my $tst_file = "examples/test.tst";
 mkdir "cache";
 mkdir "cache/htc";
 mkdir "cache/htcc";
@@ -46,6 +37,7 @@ for my $key (sort keys %use) {
     printf "using %35s %s\n", $key, $version;
 }
 HTML::Template::Compiled->clear_filecache("cache/htc");
+HTML::Template::Compiled->clear_filecache("cache/htcc");
 use Benchmark;
 my $debug = 0;
 $ENV{'HTML_TEMPLATE_ROOT'} = "examples";
@@ -55,6 +47,7 @@ my $LOOP_CONTEXT   = 1;
 my $GLOBAL_VARS    = 0;
 my $CASE_SENSITIVE = 1;
 my $default_escape = 0;
+my $template_size  = 1;
 GetOptions(
     "file-cache=i" => \$FILE_CACHE,
     "mem-cache=i" => \$MEM_CACHE,
@@ -62,18 +55,32 @@ GetOptions(
     "global-vars=i" => \$GLOBAL_VARS,
     "case-sensitive=i" => \$CASE_SENSITIVE,
     "default-escape=i" => \$default_escape,
+    "template-size=i" => \$template_size,
 );
 my $iterations = shift;
+my $ht_file = 'test.htc';
+my $htcc_file = $ht_file . 'c';
+my $tt_file = "test.tt";
+my $tst_file = "test.tst";
+$template_size =~ tr/0-9//cd;
+if ($template_size > 1) {
+    for my $file ($ht_file, $htcc_file, $tt_file, $tst_file) {
+        open my $fh, "<", "examples/$file" or die "examples/$file: $!";
+        my $data = do { local $/; <$fh> };
+        my $new_file = "$file.n$template_size";
+        open my $out, ">", "examples/$new_file" or die $!;
+        print $out $data x $template_size;
+        $file = $new_file;
+    }
+}
 
 print "running with:
-file-cache:     $FILE_CACHE
-mem-cache:      $MEM_CACHE
-loop-context:   $LOOP_CONTEXT
-global-vars:    $GLOBAL_VARS
-case-sensitive: $CASE_SENSITIVE
-default-escape: $default_escape
+--file-cache $FILE_CACHE --mem-cache $MEM_CACHE --loop-context $LOOP_CONTEXT "
+. "--global-vars $GLOBAL_VARS --case-sensitive $CASE_SENSITIVE "
+. "--default-escape $default_escape --template-size=$template_size
 ";
 
+my $STDOUT = 0;
 sub new_htc {
 	my $t1 = HTML::Template::Compiled->new_file( $ht_file,
 		#path => 'examples',
@@ -85,7 +92,7 @@ sub new_htc {
 		# first, otherwise it will run without cache
         cache_dir => ($FILE_CACHE ? "cache/htc" : undef),
         cache => $MEM_CACHE,
-		out_fh => 1,
+		out_fh => $STDOUT ? 1 : 0,
         global_vars => $GLOBAL_VARS,
         tagstyle => [qw(-asp -comment)],
 		expire_time => 1000,
@@ -104,7 +111,7 @@ sub new_htcc {
 		# first, otherwise it will run without cache
         cache_dir => ($FILE_CACHE ? "cache/htcc" : undef),
         cache => $MEM_CACHE,
-		out_fh => 1,
+		out_fh => $STDOUT ? 1 : 0,
         global_vars => $GLOBAL_VARS,
         debug => 0,
         tagstyle => [qw(-asp -comment)],
@@ -116,7 +123,7 @@ sub new_htcc {
 
 sub new_tst {
 	my $t = Text::ScriptTemplate->new();
-	$t->load($tst_file);
+	$t->load("examples/$tst_file");
 	#my $size = total_size($t1);
 	#print "size htc = $size\n";
 	return $t;
@@ -334,38 +341,49 @@ open OUT, ">>/dev/null";
 sub output {
 	my $t = shift;
 	return unless defined $t;
-	$params{name} = (ref $t).' '.$count++;
 	$t->param(%params);
-	#print $t->{code} if exists $t->{code};
-    my $out = $t=~m/Compiled/?$t->output(\*OUT):$t->output;
-    #my $out = $t=~m/Compiled/?$t->output(\*STDOUT):$t->output;
-	print OUT $out;
+    if ($STDOUT) {
+        my $out;
+        if ($t=~m/Compiled/) {
+            $out = $t->output(\*OUT);
+        }
+        else {
+            $out = $t->output;
+        }
+        print OUT $out;
+    }
+    else {
+        my $out = $t->output();
+    }
 	#print "output():$out\n";
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
-	#print "\nOUT: $out";
 }
 
-#open TT_OUT, ">&STDOUT";
 sub output_tst {
 	my $t = shift;
 	return unless defined $t;
-	#warn Data::Dumper->Dump([\%params], ['params']);
 	$t->setq(%params,tmpl=>$t);
 	my $out = $t->fill;
 	#print "output_tst():$out\n";
-	print OUT $out;
+    if ($STDOUT) {
+        print OUT $out;
+    }
 }
 sub output_tl {
 	my $t = shift;
 	return unless defined $t;
     chdir 'examples';
 	my $filett = $tt_file;
-	#$t->process($filett, \%params, \*OUT);
-	$t->process($filett, \%params, \*OUT) or die $t->error();
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
-	#print $t->{code} if exists $t->{code};
 	#my $out = $t->output;
 	#print "\nOUT: $out";
     chdir '..';
@@ -376,7 +394,13 @@ sub output_tt {
 	return unless defined $t;
 	my $filett = $tt_file;
 	#$t->process($filett, \%params, \*OUT);
-	$t->process($filett, \%params, \*OUT) or die $t->error();
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
 	#print $t->{code} if exists $t->{code};
@@ -389,7 +413,13 @@ sub output_ttaf {
 	return unless defined $t;
 	my $filett = $tt_file;
 	#$t->process($filett, \%params, \*OUT);
-	$t->process($filett, \%params, \*OUT) or die $t->error();
+    if ($STDOUT) {
+        $t->process($filett, \%params, \*OUT) or die $t->error();
+    }
+    else {
+        my $out;
+        $t->process($filett, \%params, \$out) or die $t->error();
+    }
 	#my $size = total_size($t);
 	#print "size $t = $size\n";
 	#print $t->{code} if exists $t->{code};
@@ -408,7 +438,9 @@ sub output_xslate {
 	#print $t->{code} if exists $t->{code};
     my $out = $t->render('test.xslate', \%params);
     #my $out = $t->output;
-    #print "\nOUT: $out";
+    if ($STDOUT) {
+        print OUT $out;
+    }
 }
 
 my $global_htc = $use{'HTML::Template::Compiled'} ? new_htc : undef;
